@@ -197,6 +197,90 @@ def follow_dic_order(dict_to_order,dict_to_follow):
         ordered_dic[key] = dict_to_order[key]
     return ordered_dic
 
+@njit()
+def flat_numba_drift_rebal(previous_weights, current_prices, previous_prices):
+    new_weights = previous_weights*current_prices/previous_prices
+    return new_weights
+
+@njit()
+def numba_indice_rebalancing(early_rebal, rebal_data, price_data, previous_price_data, target_weights):
+    values = np.zeros(price_data.shape[0])
+    weights = np.zeros((price_data.shape[0],price_data.shape[1]))
+    for i in range(price_data.shape[0]):
+        if i == 0:
+            weights[i, :] = target_weights
+            values[i] = np.sum(target_weights)
+            continue
+        must_rebalance = rebal_data[i]
+        if early_rebal:
+            drifted_weights = flat_numba_drift_rebal(weights[i - 1, :], price_data[i, :], previous_price_data[i, :])
+            must_rebalance = must_rebalance or np.sum(drifted_weights > target_weights)>0
+
+        if must_rebalance:
+            weights[i, :] = values[i-1]*flat_numba_drift_rebal(target_weights, price_data[i,:], previous_price_data[i,:])
+        else :
+            weights[i, :] = flat_numba_drift_rebal(weights[i - 1, :], price_data[i,:], previous_price_data[i,:])
+        values[i] = np.sum(weights[i, :])
+    return weights, values
+
+
+def pre_numba_indice_rebalancing(price_data= None, inception_date=None, target_weights=None, initial_value= 100., rebalancing_method = None , early_rebalancing = False, last = True):
+    if target_weights is None:
+        target_weights = {}
+
+    constituents = list(target_weights.keys())
+    constituents.sort()
+
+    assert len([constituent for constituent in constituents if constituent in price_data.columns]) == len(constituents)
+    total_values = sum(target_weights.values())
+    assert total_values <= 1.001
+    cash_value = 1. - total_values
+    target_weights['cash'] = cash_value
+    price_data['cash'] = 1.
+
+    constituents = list(target_weights.keys())
+    constituents.sort()
+
+    previous_prices_underlying = []
+    for me_constituent in constituents:
+        previous_prices_constituent =  f'previous_price_{me_constituent}'
+        previous_prices_underlying.append(previous_prices_constituent)
+        price_data[previous_prices_constituent] = price_data[me_constituent].shift(1)
+
+    if inception_date is None:
+        inception_date = min(price_data.index)
+
+    date_utility.add_rebalancing_datepart(price_data, 'Date', rebalancing_method = rebalancing_method, last=last)
+
+
+    print(f'backtesting with {rebalancing_method} method rebalancing since {inception_date} with early rebalancing {early_rebalancing}')
+    print(f'date range before filtering {min(price_data.index)} {max(price_data.index)}')
+    price_data = price_data.loc[price_data.index >= inception_date, :]
+    print(f'date range after filtering {min(price_data.index)} {max(price_data.index)}')
+    assert inception_date in price_data.index
+    assert price_data.shape[0] > 0
+
+
+    np_price_data = price_data[constituents].values
+    np_previous_price_data =price_data[previous_prices_underlying].values
+    np_rebal_data = price_data['is_rebalancing'].values
+
+    target_weights = np.array([target_weights[consti] for consti in constituents])
+
+    weights, indice_values = numba_indice_rebalancing(early_rebalancing, np_rebal_data, np_price_data,np_previous_price_data, target_weights)
+
+    weights_df = pd.DataFrame(data = weights, index = price_data.index, columns = constituents)
+
+    date_utility.add_rebalancing_datepart(weights_df, 'Date', rebalancing_method = rebalancing_method, last=last)
+    weights_df['indice'] = indice_values
+    for me_constituent in constituents:
+        weights_df[me_constituent] = weights_df[me_constituent] / weights_df['indice']
+
+    weights_df['indice'] = weights_df['indice'] * initial_value
+
+    return weights_df
+
+
 def pre_numba_vol_saturation_indice_rebalancing(price_data= None, inception_date=None, risk_budget_allocations = None, capped_weights=None, target_vol = None, initial_value= 100., rebalancing_method = None, volatility_window = 252, early_rebalancing = False, last = True):
     ### we must be able to assess the historical volatility on the beginning day
     assert inception_date >= min(price_data.index) + timedelta(days=volatility_window)
@@ -204,7 +288,6 @@ def pre_numba_vol_saturation_indice_rebalancing(price_data= None, inception_date
     assert capped_weights is not None
     assert target_vol is not None
 
-    #capped_weights = collections.OrderedDict(sorted(capped_weights.items()))
     capped_weights = dict(sorted(capped_weights.items(), key=lambda x: (-x[1], x[0])))
     risk_budget_allocations = follow_dic_order(risk_budget_allocations,capped_weights)
 
@@ -280,6 +363,7 @@ def pre_numba_vol_saturation_indice_rebalancing(price_data= None, inception_date
 
 
     weights_df = pd.DataFrame(data = weights_list, columns=returns_underlyings + ['cash'],index = price_data.index)
+
     weights_df['indice']=indice_values
     date_utility.add_rebalancing_datepart(weights_df, 'Date', rebalancing_method = rebalancing_method, last = last)
     return weights_df
