@@ -462,9 +462,20 @@ def to_euro(weight_data = None, rates_data = None):
 
 def to_hedged_euro(weight_data = None, rates_data = None):
     data = join_forex_data(weight_data = weight_data, rates_data = rates_data)
+
+    data['previous_EUR-USD'] = data['EUR-USD'].shift(1)
+
     data['indice_usd'] = data['indice']
+
+    data['p_n_l_dollars'] = data['indice_usd'] .pct_change()
+
     data['indice_eur'] = data['indice_usd']/data['EUR-USD']
-    data['indice'] = data['indice_eur']
+
+    data['indice_hedged_eur'] = data['indice_eur'] * data['EUR-USD'] / data['previous_EUR-USD']
+
+    data['p_n_l_eurs'] = data['indice_eur'].pct_change()
+
+    data['indice'] = data['indice_hedged_eur']
     return data
 
 
